==> raiden-0.5.1-py3-none-any.whl: raiden/ui/web/src/app/services/raiden.service.ts <==
import { HttpClient, HttpErrorResponse, HttpParams } from '@angular/common/http';
import { Injectable, NgZone } from '@angular/core';
import { bindNodeCallback, Observable, of, throwError, zip } from 'rxjs';
import { fromArray } from 'rxjs/internal/observable/fromArray';
import { catchError, combineLatest, first, flatMap, map, switchMap, tap, toArray } from 'rxjs/operators';
import { Channel } from '../models/channel';
import { Connections } from '../models/connection';
import { Event, EventsParam } from '../models/event';
import { SwapToken } from '../models/swaptoken';

import { UserToken } from '../models/usertoken';

import { RaidenConfig } from './raiden.config';
import { SharedService } from './shared.service';
import { tokenabi } from './tokenabi';

export type CallbackFunc = (error: Error, result: any) => void;

@Injectable()
export class RaidenService {

    public tokenContract: any;
    public raidenAddress: string;
    private userTokens: { [id: string]: UserToken | null } = {};

    constructor(
        private http: HttpClient,
        private zone: NgZone,
        private raidenConfig: RaidenConfig,
        private sharedService: SharedService,
    ) {
        this.tokenContract = this.raidenConfig.web3.eth.contract(tokenabi);
    }

    private zoneEncap(cb: CallbackFunc): CallbackFunc {
        return (err, res) => this.zone.run(() => cb(err, res));
    }

    get identifier(): number {
        return Math.floor(Date.now() / 1000) * 1000 + Math.floor(Math.random() * 1000);
    }

    getBlockNumber(): Observable<number> {
        return bindNodeCallback((cb: CallbackFunc) =>
            this.raidenConfig.web3.eth.getBlockNumber(this.zoneEncap(cb)))();
    }

    public checkChecksumAddress(address: string): boolean {
        return this.raidenConfig.web3.isChecksumAddress(address);
    }

    public toChecksumAddress(address: string): string {
        return this.raidenConfig.web3.toChecksumAddress(address);
    }

    public getRaidenAddress(): Observable<string> {
        return this.http.get<{ our_address: string }>(`${this.raidenConfig.api}/address`).pipe(
            map((data) => this.raidenAddress = data.our_address),
            catchError((error) => this.handleError(error)),
        );
    }

    public getChannels(): Observable<Array<Channel>> {
        return this.http.get<Array<Channel>>(`${this.raidenConfig.api}/channels`).pipe(
            flatMap((channels: Array<Channel>) => fromArray(channels)),
            flatMap((channel: Channel) => {
                return this.getUserToken(channel.token_address, false).pipe(
                    map((token: UserToken | null) => {
                        channel.userToken = token;
                        return channel;
                    })
                );
            }),
            toArray(),
            catchError((error) => this.handleError(error)),
        );
    }

    public getTokens(refresh: boolean = false): Observable<Array<UserToken>> {
        return this.http.get<Array<string>>(`${this.raidenConfig.api}/tokens`).pipe(
            combineLatest(refresh ?
                this.http.get<Connections>(`${this.raidenConfig.api}/connections`) :
                of(null)
            ),
            map(([tokenArray, connections]): Array<Observable<UserToken>> =>
                tokenArray.map((token) =>
                    this.getUserToken(token, refresh).pipe(
                        map((userToken) => userToken && connections ?
                            Object.assign(
                                userToken,
                                { connected: connections[token] }
                            ) : userToken
                        ))
                )
            ),
            switchMap((obsArray) => obsArray && obsArray.length ?
                zip(...obsArray).pipe(first()) :
                of([])
            ),
            map((tokenArray) => tokenArray.filter((token) => !!token)),
            catchError((error) => this.handleError(error)),
        );
    }

    public getChannel(tokenAddress: string, partnerAddress: string): Observable<Channel> {
        return this.http.get<Channel>(
            `${this.raidenConfig.api}/channels/${tokenAddress}/${partnerAddress}`,
        ).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public openChannel(
        tokenAddress: string,
        partnerAddress: string,
        settleTimeout: number,
        balance: number,
    ): Observable<Channel> {
        console.log('Inside the open channel service');
        const data = {
            'token_address': tokenAddress,
            'partner_address': partnerAddress,
            'settle_timeout': settleTimeout,
            'balance': balance,
        };
        return this.http.put<Channel>(`${this.raidenConfig.api}/channels`, data).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public initiatePayment(
        tokenAddress: string,
        targetAddress: string,
        amount: number,
    ): Observable<any> {
        return this.http.post(
            `${this.raidenConfig.api}/payments/${tokenAddress}/${targetAddress}`,
            { amount, identifier: this.identifier },
        ).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public depositToChannel(
        tokenAddress: string,
        partnerAddress: string,
        amount: number,
    ): Observable<Channel> {
        return this.getChannel(tokenAddress, partnerAddress).pipe(
            switchMap((channel) => this.http.patch<Channel>(
                `${this.raidenConfig.api}/channels/${tokenAddress}/${partnerAddress}`,
                { total_deposit: channel.balance + amount },
            )),
            catchError((error) => this.handleError(error)),
        );
    }

    public closeChannel(tokenAddress: string, partnerAddress: string): Observable<Channel> {
        return this.http.patch<Channel>(
            `${this.raidenConfig.api}/channels/${tokenAddress}/${partnerAddress}`,
            { state: 'closed' },
        ).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public registerToken(tokenAddress: string): Observable<UserToken> {
        return this.http.put(
            `${this.raidenConfig.api}/tokens/${tokenAddress}`,
            {},
        ).pipe(
            switchMap(() => this.getUserToken(tokenAddress).pipe(
                map((userToken) => {
                    if (userToken === null) {
                        throw new Error(`No contract on address: ${tokenAddress}`);
                    }
                    return userToken;
                }),
            )),
            catchError((error) => this.handleError(error))
        );
    }

    public connectTokenNetwork(funds: number, tokenAddress: string): Observable<any> {
        return this.http.put(
            `${this.raidenConfig.api}/connections/${tokenAddress}`,
            { funds },
        ).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public leaveTokenNetwork(tokenAddress: string): Observable<any> {
        return this.http.delete(`${this.raidenConfig.api}/connections/${tokenAddress}`).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public getEvents(
        eventsParam: EventsParam,
        fromBlock?: number,
        toBlock?: number,
    ): Observable<Array<Event>> {
        let path: string;
        const channel = eventsParam.channel;
        if (channel) {
            path = `/blockchain_events/payment_networks/${channel.token_address}/channels/${channel.partner_address}`;
        } else if (eventsParam.token) {
            path = `/blockchain_events/tokens/${eventsParam.token}`;
        } else {
            path = 'events/network';
        }
        let params = new HttpParams();
        if (fromBlock) {
            params = params.set('from_block', '' + fromBlock);
        }
        if (toBlock) {
            params = params.set('to_block', '' + toBlock);
        }
        return this.http.get<Array<Event>>(
            `${this.raidenConfig.api}/${path}`,
            { params }
        ).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public swapTokens(swap: SwapToken): Observable<boolean> {
        const data = {
            role: swap.role,
            sending_token: swap.sending_token,
            sending_amount: swap.sending_amount,
            receiving_token: swap.receiving_token,
            receiving_amount: swap.receiving_amount,
        };
        return this.http.put(
            `${this.raidenConfig.api}/token_swaps/${swap.partner_address}/${swap.identifier}`,
            data,
            { observe: 'response'},
        ).pipe(
            switchMap((response) => response.ok ? of(true) : throwError(response.toString())),
            catchError((error) => this.handleError(error)),
        );
    }

    public sha3(data: string): string {
        return this.raidenConfig.web3.sha3(data, { encoding: 'hex' });
    }

    public blocknumberToDate(block: number): Observable<Date> {
        return bindNodeCallback((b: number, cb: CallbackFunc) =>
            this.raidenConfig.web3.eth.getBlock(b, this.zoneEncap(cb))
        )(block).pipe(
            map((blk) => new Date(blk.timestamp * 1000)),
            first(),
        );
    }

    public getUserToken(
        tokenAddress: string,
        refresh: boolean = true,
    ): Observable<UserToken | null> {
        const tokenContractInstance = this.tokenContract.at(tokenAddress);
        const userToken: UserToken | null | undefined = this.userTokens[tokenAddress];

        const balanceObservable: Observable<number> = bindNodeCallback((addr: string, cb: CallbackFunc) =>
            tokenContractInstance.balanceOf(addr, this.zoneEncap(cb)),
        )(this.raidenAddress).pipe(
            map((balance) => balance.toNumber())
        );

        if (userToken === undefined) {
            const symbolObservable = bindNodeCallback((cb: CallbackFunc) =>
                tokenContractInstance.symbol(this.zoneEncap(cb))
            )();

            const nameObservable = bindNodeCallback((cb: CallbackFunc) =>
                tokenContractInstance.name(this.zoneEncap(cb))
            )();

            return zip(
                symbolObservable,
                nameObservable,
                balanceObservable,
            ).pipe(map(([symbol, name, balance]): UserToken => {
                    if (balance === null) {
                        return null;
                    }
                    return {
                        address: tokenAddress,
                        symbol,
                        name,
                        balance
                    };
                }),
                tap((token) => this.userTokens[tokenAddress] = token),
                catchError((error) => {
                    const message = (error as Error).message;
                    if (message.startsWith('Invalid JSON RPC response')) {
                        const errorMessage = 'Could not access the JSON-RPC endpoint, ' +
                            'Please make sure that CORS for this domain is enabled on your ethereum client.';
                        return throwError(Error(errorMessage));
                    }
                    return throwError(error);
                })
            );

        } else if (refresh && userToken !== null) {
            return balanceObservable.pipe(
                map((balance) => {
                    if (balance === null) {
                        return null;
                    }
                    userToken.balance = balance;
                    return userToken;
                }),
            );
        } else {
            return of(userToken);
        }
    }

    private handleError(error: Response | Error | any) {
        // In a real world app, you might use a remote logging infrastructure
        let errMsg: string;
        if (error instanceof Response) {
            let body;
            try {
                body = error.json() || '';
            } catch (e) {
                body = error.text();
            }
            const err = body || JSON.stringify(body);
            errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
        } else if (error instanceof HttpErrorResponse && error.error['errors']) {
            const errors = error.error.errors;

            if (typeof errors === 'string') {
                errMsg = errors;
            } else if (typeof errors === 'object') {
                errMsg = '';

                for (const key in errors) {
                    if (errors.hasOwnProperty(key)) {
                        if (errMsg !== '') {
                            errMsg += '\n';
                        }
                        errMsg += `${key}: ${errors[key]}`;
                    }
                }

            } else {
                errMsg = errors;
            }

        } else {
            errMsg = error.message ? error.message : error.toString();
        }
        console.error(errMsg);
        this.sharedService.msg({
            severity: 'error',
            summary: 'Raiden Error',
            detail: typeof errMsg === 'string' ? errMsg : JSON.stringify(errMsg),
        });
        return throwError(errMsg);
    }
}

==> raiden-0.7.0-py3-none-any.whl: raiden/ui/web/src/app/services/raiden.service.ts <==
import { HttpClient, HttpErrorResponse, HttpParams } from '@angular/common/http';
import { Injectable, NgZone } from '@angular/core';
import { bindNodeCallback, combineLatest, from, Observable, of, throwError, zip } from 'rxjs';
import { catchError, first, flatMap, map, share, switchMap, tap, toArray } from 'rxjs/operators';
import { Channel } from '../models/channel';
import { Connections } from '../models/connection';
import { Event, EventsParam } from '../models/event';
import { SwapToken } from '../models/swaptoken';

import { UserToken } from '../models/usertoken';
import { amountFromDecimal, amountToDecimal } from '../utils/amount.converter';

import { RaidenConfig } from './raiden.config';
import { SharedService } from './shared.service';
import { tokenabi } from './tokenabi';
import { BigNumber } from 'bignumber.js';

export type CallbackFunc = (error: Error, result: any) => void;

@Injectable({
    providedIn: 'root'
})
export class RaidenService {

    public tokenContract: any;
    public raidenAddress: string;
    private userTokens: { [id: string]: UserToken | null } = {};

    private defaultDecimals = 18;

    constructor(
        private http: HttpClient,
        private zone: NgZone,
        private raidenConfig: RaidenConfig,
        private sharedService: SharedService,
    ) {
        this.tokenContract = this.raidenConfig.web3.eth.contract(tokenabi);
    }

    // noinspection JSMethodCanBeStatic

    get identifier(): number {
        return Math.floor(Date.now() / 1000) * 1000 + Math.floor(Math.random() * 1000);
    }

    getBlockNumber(): Observable<number> {
        return bindNodeCallback((cb: CallbackFunc) =>
            this.raidenConfig.web3.eth.getBlockNumber(this.zoneEncap(cb)))();
    }

    public checkChecksumAddress(address: string): boolean {
        return this.raidenConfig.web3.isChecksumAddress(address);
    }

    public toChecksumAddress(address: string): string {
        return this.raidenConfig.web3.toChecksumAddress(address);
    }

    public getRaidenAddress(): Observable<string> {
        return this.http.get<{ our_address: string }>(`${this.raidenConfig.api}/address`).pipe(
            map((data) => this.raidenAddress = data.our_address),
            catchError((error) => this.handleError(error)),
        );
    }

    public getChannels(): Observable<Array<Channel>> {
        return this.http.get<Array<Channel>>(`${this.raidenConfig.api}/channels`).pipe(
            flatMap((channels: Array<Channel>) => from(channels)),
            flatMap((channel: Channel) => {
                return this.getUserToken(channel.token_address, false).pipe(
                    map((token: UserToken | null) => {
                        channel.userToken = token;
                        return channel;
                    })
                );
            }),
            toArray(),
            catchError((error) => this.handleError(error)),
        );
    }

    public getTokens(refresh: boolean = false): Observable<Array<UserToken>> {
        const tokens$ = this.http.get<Array<string>>(`${this.raidenConfig.api}/tokens`);
        const connections$ = refresh ?
            this.http.get<Connections>(`${this.raidenConfig.api}/connections`) :
            of(null);

        return combineLatest(tokens$, connections$).pipe(
            map(([tokenArray, connections]): Array<Observable<UserToken>> =>
                tokenArray.map((token) =>
                    this.getUserToken(token, refresh).pipe(
                        map((userToken) => userToken && connections ?
                            Object.assign(
                                userToken,
                                {connected: connections[token]}
                            ) : userToken
                        ))
                )
            ),
            switchMap((obsArray) => obsArray && obsArray.length ?
                zip(...obsArray).pipe(first()) :
                of([])
            ),
            map((tokenArray) => tokenArray.filter((token) => !!token)),
            catchError((error) => this.handleError(error)),
        );
    }

    public getChannel(tokenAddress: string, partnerAddress: string): Observable<Channel> {
        return this.http.get<Channel>(
            `${this.raidenConfig.api}/channels/${tokenAddress}/${partnerAddress}`,
        ).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public openChannel(
        tokenAddress: string,
        partnerAddress: string,
        settleTimeout: number,
        balance: number,
        decimals: number
    ): Observable<Channel> {
        const data = {
            'token_address': tokenAddress,
            'partner_address': partnerAddress,
            'settle_timeout': settleTimeout,
            'total_deposit': amountFromDecimal(balance, decimals),
        };
        return this.http.put<Channel>(`${this.raidenConfig.api}/channels`, data).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public initiatePayment(
        tokenAddress: string,
        targetAddress: string,
        amount: number,
        decimals: number
    ): Observable<any> {
        const raidenAmount = amountFromDecimal(amount, decimals);

        return this.http.post(
            `${this.raidenConfig.api}/payments/${tokenAddress}/${targetAddress}`,
            {
                amount: raidenAmount,
                identifier: this.identifier
            },
        ).pipe(
            tap((response) => {
                if ('target_address' in response && 'identifier' in response) {
                    const formattedAmount = amount.toFixed(decimals).toString();
                    this.sharedService.success({
                        title: 'Transfer successful',
                        description: `A payment of ${formattedAmount} was successfully sent to the partner ${targetAddress}`
                    });
                } else {
                    this.sharedService.error({
                        title: 'Payment error',
                        description: JSON.stringify(response),
                    });
                }
            }),
            catchError((error) => this.handleError(error)),
        );
    }

    public depositToChannel(
        tokenAddress: string,
        partnerAddress: string,
        amount: number,
        decimals: number
    ): Observable<Channel> {

        const depositIncrement = amountFromDecimal(amount, decimals);

        return this.getChannel(tokenAddress, partnerAddress).pipe(
            switchMap((channel) => this.http.patch<Channel>(
                `${this.raidenConfig.api}/channels/${tokenAddress}/${partnerAddress}`,
                {total_deposit: channel.total_deposit + depositIncrement},
            )),
            tap((response) => {
                const action = 'Deposit';
                if ('balance' in response && 'state' in response) {
                    const balance = amountToDecimal(response.balance, decimals);
                    const formattedBalance = balance.toFixed(decimals).toString();
                    this.sharedService.info({
                        title: action,
                        description: `The channel ${response.channel_identifier} has been modified with a deposit of ${formattedBalance}`
                    });
                } else {
                    this.sharedService.error({
                        title: action,
                        description: JSON.stringify(response)
                    });
                }
            }),
            catchError((error) => this.handleError(error)),
        );
    }

    public closeChannel(tokenAddress: string, partnerAddress: string): Observable<Channel> {
        return this.http.patch<Channel>(
            `${this.raidenConfig.api}/channels/${tokenAddress}/${partnerAddress}`,
            {state: 'closed'},
        ).pipe(
            tap(response => {
                const action = 'Close';
                if ('state' in response && response.state === 'closed') {
                    this.sharedService.info({
                        title: action,
                        description: `The channel ${response.channel_identifier} with partner
                    ${response.partner_address} has been closed successfully`
                    });
                } else {
                    this.sharedService.error({
                        title: action,
                        description: JSON.stringify(response)
                    });
                }
            }),
            catchError((error) => this.handleError(error)),
        );
    }

    public registerToken(tokenAddress: string): Observable<UserToken> {
        return this.http.put(
            `${this.raidenConfig.api}/tokens/${tokenAddress}`,
            {},
        ).pipe(
            switchMap(() => this.getUserToken(tokenAddress).pipe(
                map((userToken) => {
                    if (userToken === null) {
                        throw new Error(`No contract on address: ${tokenAddress}`);
                    }
                    return userToken;
                }),
            )),
            tap((userToken: UserToken) => {
                this.sharedService.success({
                    title: 'Token registered',
                    description: `Your token was successfully registered: ${userToken.address}`
                });
            }),
            catchError((error) => this.handleError(error))
        );
    }

    public connectTokenNetwork(funds: number, tokenAddress: string, decimals: number): Observable<any> {
        return this.http.put(
            `${this.raidenConfig.api}/connections/${tokenAddress}`,
            {funds: amountFromDecimal(funds, decimals)},
        ).pipe(
            tap(() => {
                this.sharedService.success({
                    title: 'Joined Token Network',
                    description: `You have successfully Joined the Network of Token ${tokenAddress}`
                });
            }),
            catchError((error) => this.handleError(error)),
        );
    }

    public leaveTokenNetwork(userToken: UserToken): Observable<any> {
        return this.http.delete(`${this.raidenConfig.api}/connections/${userToken.address}`).pipe(
            map(() => true),
            tap(() => {
                const description = `Successfully closed and settled all channels in ${userToken.name} <${userToken.address}> token`;
                this.sharedService.success({
                    title: 'Left Token Network',
                    description: description,
                });
            }),
            catchError((error) => this.handleError(error)),
        );
    }

    public getBlockchainEvents(
        eventsParam: EventsParam,
        fromBlock?: number,
        toBlock?: number,
    ): Observable<Array<Event>> {
        let path: string;
        const channel = eventsParam.channel;
        const basePath = '_debug/blockchain_events';

        if (channel) {
            path = `${basePath}/payment_networks/${channel.token_address}/channels/${channel.partner_address}`;
        } else if (eventsParam.token) {
            path = `${basePath}/tokens/${eventsParam.token}`;
        } else {
            path = `${basePath}/network`;
        }
        let params = new HttpParams();
        if (fromBlock) {
            params = params.set('from_block', '' + fromBlock);
        }
        if (toBlock) {
            params = params.set('to_block', '' + toBlock);
        }
        return this.http.get<Array<Event>>(
            `${this.raidenConfig.api}/${path}`,
            {params}
        ).pipe(
            catchError((error) => this.handleError(error)),
        );
    }

    public swapTokens(swap: SwapToken): Observable<boolean> {
        const data = {
            role: swap.role,
            sending_token: swap.sending_token,
            sending_amount: swap.sending_amount,
            receiving_token: swap.receiving_token,
            receiving_amount: swap.receiving_amount,
        };
        return this.http.put(
            `${this.raidenConfig.api}/token_swaps/${swap.partner_address}/${swap.identifier}`,
            data,
            {observe: 'response'},
        ).pipe(
            switchMap((response) => response.ok ? of(true) : throwError(response.toString())),
            catchError((error) => this.handleError(error)),
        );
    }

    public blocknumberToDate(block: number): Observable<Date> {
        return bindNodeCallback((b: number, cb: CallbackFunc) =>
            this.raidenConfig.web3.eth.getBlock(b, this.zoneEncap(cb))
        )(block).pipe(
            map((blk) => new Date(blk.timestamp * 1000)),
            first(),
        );
    }

    public getUserToken(
        tokenAddress: string,
        refresh: boolean = true,
    ): Observable<UserToken | null> {
        const tokenContractInstance = this.tokenContract.at(tokenAddress);
        const tokenMap = this.userTokens;
        const userToken: UserToken | null | undefined = tokenMap[tokenAddress];

        const balance$: Observable<number> = bindNodeCallback((addr: string, cb: CallbackFunc) =>
            tokenContractInstance.balanceOf(addr, this.zoneEncap(cb)),
        )(this.raidenAddress).pipe(
            map((balance) => balance.toNumber())
        );

        if (userToken === undefined) {
            const decimals$: Observable<BigNumber> = bindNodeCallback((cb: CallbackFunc) =>
                tokenContractInstance.decimals(this.zoneEncap(cb))
            )().pipe(catchError(() => of(this.defaultDecimals)));

            const symbol$: Observable<string> = bindNodeCallback((cb: CallbackFunc) =>
                tokenContractInstance.symbol(this.zoneEncap(cb))
            )();

            const name$: Observable<string> = bindNodeCallback((cb: CallbackFunc) =>
                tokenContractInstance.name(this.zoneEncap(cb))
            )();

            return zip(
                symbol$,
                name$,
                balance$,
                decimals$
            ).pipe(
                map(([symbol, name, balance, decimals]): UserToken => {
                    return ({
                        address: tokenAddress,
                        symbol,
                        name,
                        balance,
                        decimals: decimals.toNumber()
                    });
                }),
                tap((token) => tokenMap[tokenAddress] = token),
                share(),
                catchError((error) => {
                    const message = (error as Error).message;
                    if (message.startsWith('Invalid JSON RPC response')) {
                        const errorMessage = 'Could not access the JSON-RPC endpoint, ' +
                            'Please make sure that CORS for this domain is enabled on your ethereum client.';
                        return throwError(Error(errorMessage));
                    }
                    return throwError(error);
                })
            );

        } else if (refresh && userToken !== null) {
            return balance$.pipe(
                map((balance) => {
                    if (balance === null) {
                        return null;
                    }
                    userToken.balance = balance;
                    return userToken;
                }),
            );
        } else {
            return of(userToken);
        }
    }

    private zoneEncap(cb: CallbackFunc): CallbackFunc {
        return (err, res) => this.zone.run(() => cb(err, res));
    }

    private handleError(error: Response | Error | any) {
        // In a real world app, you might use a remote logging infrastructure
        let errMsg: string;
        if (error instanceof Response) {
            let body;
            try {
                body = error.json() || '';
            } catch (e) {
                body = error.text();
            }
            const err = body || JSON.stringify(body);
            errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
        } else if (error instanceof HttpErrorResponse && error.error['errors']) {
            const errors = error.error.errors;

            if (typeof errors === 'string') {
                errMsg = errors;
            } else if (typeof errors === 'object') {
                errMsg = '';

                for (const key in errors) {
                    if (errors.hasOwnProperty(key)) {
                        if (errMsg !== '') {
                            errMsg += '\n';
                        }
                        errMsg += `${key}: ${errors[key]}`;
                    }
                }

            } else {
                errMsg = errors;
            }

        } else {
            errMsg = error.message ? error.message : error.toString();
        }
        console.error(errMsg);
        this.sharedService.error({
            title: 'Raiden Error',
            description: typeof errMsg === 'string' ? errMsg : JSON.stringify(errMsg)
        });
        return throwError(errMsg);
    }
}
